@@ -14,7 +14,9 @@
 r"""ATBU Configuration-related classes/functions.
 - AtbuConfig is the primary class.
 """
+import base64
 import fnmatch
+import glob
 import logging
 import os
 from pathlib import Path
@@ -139,6 +141,26 @@ def add_path_value(
         value_cfg = value_cfg[s]
     value_cfg[section_value_path[-1]] = value
 
+def prompt_user_password_file_does_not_exist(default_filename: str):
+    is_exist = default_filename is not None and os.path.isfile(default_filename)
+    print(
+        f"Specify a path to an existing password .json file."
+    )
+    def_qual = ""
+    if is_exist:
+        def_qual = "<ENTER> for the default, "
+        print(
+            f"If you do not specify a name and press <ENTER>, the default name will be used."
+        )
+        print(
+            f"Default path (file exists): {default_filename}"
+        )
+    a = input(f"Enter the path, {def_qual}or 'n' to abort:")
+    if a.lower() in ["n", "no"]:
+        return None
+    if is_exist and a == "":
+        return default_filename
+    return a
 
 def restore_keyring_secrets(storage_def: dict):
     keyring_section: dict = storage_def.get(CONFIG_SECTION_KEYRING_MAPPING)
@@ -160,6 +182,30 @@ def restore_keyring_secrets(storage_def: dict):
         cba_password = CredentialByteArray(
             affected_section[affected_config_path_parts[-1]].encode("utf-8")
         )
+
+        if password_type == CONFIG_PASSWORD_TYPE_FILENAME:
+            filename = base64.b64decode(cba_password.decode("utf-8")).decode("utf-8")
+            while not os.path.isfile(filename):
+                print(
+                    f"The credential file does not exist: {filename}"
+                )
+                user_specified_filename = prompt_user_password_file_does_not_exist(filename)
+                if user_specified_filename is None:
+                    raise CredentialSecretFileNotFoundError(
+                        f"An existing credential password filename was not specified, aborting restore."
+                    )
+                if os.path.isfile(user_specified_filename):
+                    cba_password = CredentialByteArray(
+                        base64.b64encode(user_specified_filename.encode())
+                    )
+                    break
+                print(
+                    f"ERROR: The filename entered does not exist: {user_specified_filename}"
+                )
+                print(
+                    f"You must specify a path to an existing .json password file (i.e., a service account .json OAuth2 file)."
+                )
+
         set_password_to_keyring(
             service_name=service_name,
             username=username,
@@ -462,6 +508,13 @@ configuration, you can choose to have one created for you.
         self.save_config_file()
         return backup_info_dir
 
+    def get_backup_info_file_paths(
+        self,
+        storage_def_name: str,
+    ) -> list[str]:
+        pattern = self.get_backup_info_dir() / f"{storage_def_name}*"
+        return glob.glob(pathname=str(pattern))
+
     def resolve_storage_location(
         self,
         storage_location,
