@@ -583,6 +583,16 @@ fail to properly backup your locally stored backup information (which includes t
         default=False,
         help="Delete without a confirmation prompt.",
     )
+    cred_delete_storage_def_parser.add_argument(
+        "--delete-backup-info", "--dbi",
+        action=argparse.BooleanOptionalAction,
+        help="""If you specify --no-delete-backup-info (--no-dbi), backup information files
+will not be deleted. By default, a storage definition's backup informatiion
+files are deleted along with the storage definition itself. If this option
+is used without --force, prompting for deleting backup information files
+will be skipped (i.e., user only prompted once re storage defintiion).
+""",
+    )
 
     #
     # creds export:
